from pydantic import BaseModel
from typing import Optional


class TargetApp(BaseModel):
    app: str
    version: Optional[str] = "_"
    branch: Optional[str] = "_"

    def to_slashed_string(self):
        return f"{self.app}/{self.version}/{self.branch}"

    def to_provider_string(self):
        return f"{self.app}/{self.version}"

    @staticmethod
    def from_unique_slashed_string(slashed_string: str):
        target = slashed_string.split("/")
        if len(target) == 3:
            return TargetApp(app=target[0], version=target[1], branch=target[2])
        elif len(target) == 2:
            return TargetApp(app=target[0], version=target[1], branch="_")
        elif len(target) == 1:
            return TargetApp(app=target[0], version="_", branch="_")





import contextlib
import http

import tenacity
from httpx import (
    AsyncClient,
    Timeout,
    HTTPError,
    Response,
    TimeoutException,
    UnsupportedProtocol,
    InvalidURL,
)
from typing import Any, AsyncContextManager, AsyncIterator, Dict, Callable, Sequence
from typing import Optional
import logging

from shared_dependencies.sentry import safe_capture_exception

# default_retry_policy : hook pour get ou post
from tenacity import (
    retry_if_not_exception_type,
    stop_after_delay,
    wait_random_exponential,
    AsyncRetrying,
)

try:
    # optionnal dependency
    # if fastapi is installed, then it's better to use their HTTPException
    # so that it gets treated as a proper response by the framework
    # if fastapi is not part of the project, then a very similar Exception Class will be used (functionnaly the same)
    # cf https://github.com/encode/starlette/blob/master/starlette/exceptions.py
    # and https://github.com/tiangolo/fastapi/blob/22528373bba6a654323de416ad5c867cbadb81bb/fastapi/exceptions.py
    from fastapi import HTTPException  # noqa: E402
except ImportError:

    class HTTPException(Exception):  # type: ignore
        def __init__(
            self,
            status_code: int,
            detail: Any = None,
            headers: Optional[Dict[str, Any]] = None,
        ) -> None:
            self.headers = headers
            if detail is None:
                detail = http.HTTPStatus(status_code).phrase
            self.status_code = status_code
            self.detail = detail

    def __repr__(self) -> str:
        class_name = self.__class__.__name__
        return f"{class_name}(status_code={self.status_code!r}, detail={self.detail!r})"


logger = logging.getLogger(__name__)


async def default_log_request(request):
    logger.info(f"Request event hook: {request.method} {request.url} - Waiting for response")


async def default_log_response(response):
    request = response.request
    logger.info(
        f"Response event hook: {request.method} {request.url} - Status {response.status_code}"
    )


default_event_hooks = {"request": [default_log_request], "response": [default_log_response]}
default_retrier = AsyncRetrying(
    reraise=True,
    retry=retry_if_not_exception_type((TimeoutException, UnsupportedProtocol, InvalidURL)),
    stop=stop_after_delay(60),
    wait=wait_random_exponential(max=60, multiplier=2),
    after=tenacity.after_log(logger, logging.INFO),
)


def default_should_retry(*args, **kwargs):
    context = kwargs["context"]
    if context == "before":
        return kwargs.get("method", "").upper() == "GET"
    if context == "after":
        return args[0].status_code in {502, 503, 504, 507, 509}


class CustomAsyncClient:
    def __init__(
        self,
        timeout: float = 60.0,
        raise_if_4xx_5xx: bool = True,
        should_retry: Optional[Callable] = None,
        retry_policy: Optional[AsyncRetrying] = None,
        should_raise_error_as_httpexception: Optional[Callable] = None,
        event_hooks: Dict[str, Sequence[Callable]] = None,
        *args: Any,
        **kwargs: Any,
    ) -> None:
        self.timeout = timeout
        self.raise_if_4xx_5xx = raise_if_4xx_5xx
        self.client: AsyncContextManager = lambda: self.get_client(*args, **kwargs)
        self.should_retry = default_should_retry if should_retry is None else should_retry
        self.retry_policy = default_retrier if retry_policy is None else retry_policy
        self.event_hooks = default_event_hooks if event_hooks is None else event_hooks
        if should_raise_error_as_httpexception is None:
            self.should_raise_error_as_httpexception = lambda e: True
        else:
            self.should_raise_error_as_httpexception = should_raise_error_as_httpexception

    @contextlib.asynccontextmanager
    async def get_client(self, *args: Any, **kwargs: Any) -> AsyncIterator:
        async with AsyncClient(
            *args,
            timeout=Timeout(timeout=self.timeout, connect=self.timeout),
            event_hooks=self.event_hooks,
            **kwargs,
        ) as client:
            yield client

    async def request(self, *args, **kwargs) -> Response:
        async with self.client() as client:
            try:
                if self.should_retry(*args, **kwargs, context="before"):
                    res = await self.request_with_retry(client, *args, **kwargs)
                else:
                    res = await self.request_no_retry(client, *args, **kwargs)
                if self.raise_if_4xx_5xx:
                    res.raise_for_status()  # HTTPStatusError (inherits HTTPError) if 4xx or 5xx
            except HTTPError as e:
                self.log_and_raise_exception(e)
            return res

    async def request_no_retry(self, client, *args, **kwargs):
        res = await client.request(*args, **kwargs)
        return res

    async def request_with_retry(self, client, *args, **kwargs):
        async for attempt in self.retry_policy:
            with attempt:
                res = await client.request(*args, **kwargs)
                if self.should_retry(res, context="after"):
                    res.raise_for_status()
        return res

    def log_and_raise_exception(self, e: HTTPError) -> Response:
        has_response = hasattr(e, "response")
        status_code = e.response.status_code if has_response else 500
        message = e.response.text if has_response else str(e)
        # attempt to use agathe code here
        event_id = safe_capture_exception(e)
        json_content = {"message": message, "sentry_event_id": event_id}
        if self.should_raise_error_as_httpexception(e):
            raise HTTPException(
                status_code=status_code,
                detail=json_content,  # mettre le code d'Agathe ici
            )
        else:
            raise e

    async def get(self, *args, **kwargs):
        url, *new_args = args
        return await self.request(method="get", url=url, *new_args, **kwargs)

    async def post(self, *args, **kwargs):
        url, *new_args = args
        return await self.request(method="post", url=url, *new_args, **kwargs)

    async def put(self, *args, **kwargs):
        url, *new_args = args
        return await self.request(method="put", url=url, *new_args, **kwargs)

    async def patch(self, *args, **kwargs):
        url, *new_args = args
        return await self.request(method="put", url=url, *new_args, **kwargs)

    async def head(self, *args, **kwargs):
        url, *new_args = args
        return await self.request(method="head", url=url, *new_args, **kwargs)

    async def options(self, *args, **kwargs):
        url, *new_args = args
        return await self.request(method="options", url=url, *new_args, **kwargs)

    async def delete(self, *args, **kwargs):
        url, *new_args = args
        return await self.request(method="delete", url=url, *new_args, **kwargs)



import os
import contextlib
import json
from abc import ABC, abstractmethod


@contextlib.contextmanager
def tempenv(**update):
    """
    Temporarily updates the ``os.environ``
    :param update: Dictionary of environment variables and values to add/update.
    """
    env = os.environ
    previous_values = {k: env.get(k, None) for k in update}
    for k, v in update.items():
        env[k] = v
    yield
    for k, v in previous_values.items():
        if v is None:
            # means that it was not an existing key before the updating operation
            env.pop(k)
        else:
            env[k] = v


class CachingEngine(ABC):
    @abstractmethod
    def set(self, key: str, val: str):
        pass

    @abstractmethod
    def get_cache_or_none(self, key: str):
        pass


class Cached:
    """cache a function that returns jsonifiable output"""

    def __init__(self, cache_key_func, caching_engine: CachingEngine, should_cache=None):
        self.cache_key_func = cache_key_func
        self.caching_engine = caching_engine
        self._should_cache = should_cache

    def should_cache(self, *args, **kwargs):
        if self._should_cache is None:
            return True
        else:
            return self._should_cache(*args, **kwargs)

    def __call__(self, function_to_be_decorated):
        def new_func(*args, **kwargs):
            key = self.cache_key_func(*args, **kwargs)
            cached_value = self.caching_engine.get_cache_or_none(key)
            if cached_value is None:
                val = function_to_be_decorated(*args, **kwargs)
                if self.should_cache(*args, value_to_cache=val, **kwargs):
                    self.caching_engine.set(key, json.dumps(val))
                return val
            else:
                return cached_value

        return new_func


class AsyncCached:
    """cache a function that returns jsonifiable output"""

    def __init__(self, cache_key_func, caching_engine: CachingEngine, should_cache=None):
        self.cache_key_func = cache_key_func
        self.caching_engine = caching_engine
        self._should_cache = should_cache

    def should_cache(self, *args, **kwargs):
        if self._should_cache is None:
            return True
        else:
            return self._should_cache(*args, **kwargs)

    def __call__(self, function_to_be_decorated):
        async def new_func(*args, **kwargs):
            key = self.cache_key_func(*args, **kwargs)
            cached_value = self.caching_engine.get_cache_or_none(key)
            if cached_value is None:
                val = await function_to_be_decorated(*args, **kwargs)
                if self.should_cache(*args, value_to_cache=val, **kwargs):
                    self.caching_engine.set(key, json.dumps(val))
                return val
            else:
                return cached_value

        return new_func


class RedisCachingEngine(CachingEngine):
    def __init__(self, redis_client, exp):
        self.redis_client = redis_client
        self.exp = exp

    def get_cache_or_none(self, key):
        cache_value = self.redis_client.get(key)
        if cache_value is None:
            return None
        else:
            return json.loads(cache_value)

    def set(self, k, v):
        self.redis_client.set(k, v, ex=self.exp)


class RedisCached(Cached):
    def __init__(self, redis_client, key_func, exp, should_cache=None):
        return super().__init__(key_func, RedisCachingEngine(redis_client, exp), should_cache)


class AsyncRedisCached(AsyncCached):
    def __init__(self, redis_client, key_func, exp, should_cache=None):
        return super().__init__(key_func, RedisCachingEngine(redis_client, exp), should_cache)



import os
import logging
from typing import Callable, List, Union, Dict, Any, Sequence

from sentry_sdk import capture_exception, init
from sentry_sdk.integrations import Integration
from sentry_sdk.integrations.pure_eval import PureEvalIntegration
from sentry_sdk.integrations.logging import LoggingIntegration

logger = logging.getLogger("uvicorn")

sentry_logging = LoggingIntegration(level=logging.INFO, event_level=logging.ERROR)

common_integrations = [
    sentry_logging,
    PureEvalIntegration(),
]


def safe_capture_exception(e: Exception) -> str:
    sentry_id_or_none = capture_exception(e)
    return sentry_id_or_none if sentry_id_or_none is not None else "not sent"


def init_sentry(sentry_integrations: Sequence[Integration]):
    """
    This method has to be called in the app init file (main.py).
    SENTRY_DSN, SENTRY_ENVIRONMENT and SENTRY_RELEASE must be provided in app variables.
    :param sentry_integrations:
    :return: sentry init function
    """
    return init(
        integrations=sentry_integrations,
        traces_sample_rate=float(os.getenv("TRACES_SAMPLE_RATE_SENTRY", 1.0)),
        sample_rate=float(os.getenv("SAMPLE_RATE_SENTRY", 1.0)),
        traces_sampler=sampling_context,
    )  # type: ignore


def sampling_context(context: Dict[str, Any]) -> Union[int, float]:
    try:
        if "/metrics" in context["asgi_scope"].get("path", ""):
            return 0
    except KeyError:
        logger.exception("context->asgi_scope failed")
        return 0
    else:
        logger.info(f'sentry-trace : {context["transaction_context"]["trace_id"]}')
        return float(os.getenv("TRACES_SAMPLE_RATE_SENTRY", 1.0))



import os
import contextlib
import jwt
from jwt.exceptions import PyJWTError, DecodeError, ExpiredSignatureError
import base64
from math import ceil, log
from Crypto.Cipher import AES
from typing import Any
from typing import Dict
from datetime import datetime, timedelta


BLOCK_SIZE = AES.block_size


def pad(s: str) -> str:
    return s + (BLOCK_SIZE - len(s) % BLOCK_SIZE) * chr(BLOCK_SIZE - len(s) % BLOCK_SIZE)


def unpad(s: bytes) -> bytes:
    return s[: -ord(s[len(s) - 1 :])]


def encrypt(raw: str, password: str) -> bytes:
    private_key = password.encode("utf-8")
    raw = pad(raw)
    cipher = AES.new(private_key, AES.MODE_ECB)
    return base64.b64encode(cipher.encrypt(raw))


def decrypt(enc: str, password: str) -> bytes:
    private_key = password.encode("utf-8")
    enc_bytes = base64.b64decode(enc)
    cipher = AES.new(private_key, AES.MODE_ECB)
    return unpad(cipher.decrypt(enc_bytes))


def read_key_from_env(key: str, linebreakword: str = "_LINEBREAK_") -> str:
    content = os.getenv(key)
    if content is None:
        raise ValueError(f"{key} not found in env variables")
    if content.startswith(linebreakword):
        content = content[len(linebreakword) :]
    if content.endswith(linebreakword):
        content = content[: -len(linebreakword)]
    return content.replace(linebreakword, "\n")


def decrypt_aes_encoding(token: str, secret: str) -> str:
    try:
        return decrypt(token, secret).decode("utf-8")
    except ValueError:
        raise Exception("Impossible to decode token with given secret")


def ensure_token_validity(token: str) -> None:
    public_key = read_key_from_env("PUBLIC_KEY")
    try:
        payload = jwt.decode(
            token,
            public_key,
            algorithms=["RS256"],
        )
        if "sub" not in payload:
            raise Exception("sub key is missing")
    except DecodeError:
        raise Exception("token is not an encoded JWT")
    except ExpiredSignatureError:
        raise Exception("token is expired")
    except Exception:
        raise Exception("unknown error")


def get_payload(token: str, safe: bool = True) -> Dict[str, Any]:
    if safe:
        ensure_token_validity(token)
    return jwt.decode(
        token,
        read_key_from_env("PUBLIC_KEY"),
        algorithms=["RS256"],
    )


def nanoid_algorithm(random_bytes: int) -> bytearray:
    """From https://github.com/puyuan/py-nanoid"""
    return bytearray(os.urandom(random_bytes))


def generate_session_id(
    alphabet: str = "_-0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ",
    size: int = 8,
) -> str:
    """Adapted from https://github.com/puyuan/py-nanoid"""
    alphabet_len = len(alphabet)
    mask = 1
    if alphabet_len > 1:
        mask = (2 << int(log(alphabet_len - 1) / log(2))) - 1
    step = int(ceil(1.6 * mask * size / alphabet_len))
    session_id = ""
    while True:
        random_bytes = nanoid_algorithm(step)
        for i in range(step):
            random_byte = random_bytes[i] & mask
            if random_byte < alphabet_len and alphabet[random_byte]:
                session_id += alphabet[random_byte]
                if len(session_id) == size:
                    return session_id


def generate_token(token_info: dict, exp_min: int):
    exp_date = datetime.now() + timedelta(minutes=exp_min)
    exp = {"exp": datetime.timestamp(exp_date)}
    jwt_data = jwt.encode(
        {**token_info, **exp},
        read_key_from_env("PRIVATE_KEY"),
        algorithm="RS256",
    )
    return encrypt(jwt_data, os.getenv("AES_SECRET")).decode("utf-8")


def get_admin_token():
    return generate_token(token_info={"sub": "ms_auth"}, exp_min=15)


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['shared_dependencies']

package_data = \
{'': ['*']}

install_requires = \
['PyJWT>=2.3.0,<3.0.0',
 'asttokens>=2.0.5,<3.0.0',
 'colorama>=0.4.4,<0.5.0',
 'cryptography>=36.0.1,<37.0.0',
 'executing>=0.8.2,<0.9.0',
 'httpx>=0.21.1,<0.22.0',
 'pure-eval>=0.2.1,<0.3.0',
 'pycrypto>=2.6.1,<3.0.0',
 'pydantic[dotenv]>=1.9.0,<2.0.0',
 'sentry-sdk>=1.5.1,<2.0.0',
 'tenacity>=8.0.1,<9.0.0']

setup_kwargs = {
    'name': 'shared-dependencies',
    'version': '0.1.22',
    'description': '',
    'long_description': None,
    'author': 'Jean-Charles Bouchaud',
    'author_email': 'jeancharles-b@evidenceb.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
